@@ -1,5 +1,5 @@
 /* BFD library support routines for architectures.
-   Copyright (C) 1990-2019 Free Software Foundation, Inc.
+   Copyright (C) 1990-2026 Free Software Foundation, Inc.
    Hacked by Steve Chamberlain of Cygnus Support.
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -23,6 +23,7 @@
 #include "bfd.h"
 #include "libbfd.h"
 #include "opcode/m68k.h"
+#include "cpu-m68k.h"
 
 static const bfd_arch_info_type *
 bfd_m68k_compatible (const bfd_arch_info_type *a,
@@ -34,79 +35,79 @@ bfd_m68k_compatible (const bfd_arch_info_type *a,
 
 static const bfd_arch_info_type arch_info_struct[] =
   {
-    N(bfd_mach_m68000,  "m68k:68000", FALSE, &arch_info_struct[1]),
-    N(bfd_mach_m68008,  "m68k:68008", FALSE, &arch_info_struct[2]),
-    N(bfd_mach_m68010,  "m68k:68010", FALSE, &arch_info_struct[3]),
-    N(bfd_mach_m68020,  "m68k:68020", FALSE, &arch_info_struct[4]),
-    N(bfd_mach_m68030,  "m68k:68030", FALSE, &arch_info_struct[5]),
-    N(bfd_mach_m68040,  "m68k:68040", FALSE, &arch_info_struct[6]),
-    N(bfd_mach_m68060,  "m68k:68060", FALSE, &arch_info_struct[7]),
-    N(bfd_mach_cpu32,   "m68k:cpu32", FALSE, &arch_info_struct[8]),
-    N(bfd_mach_fido,    "m68k:fido",  FALSE, &arch_info_struct[9]),
+    N(bfd_mach_m68000,  "m68k:68000", false, &arch_info_struct[1]),
+    N(bfd_mach_m68008,  "m68k:68008", false, &arch_info_struct[2]),
+    N(bfd_mach_m68010,  "m68k:68010", false, &arch_info_struct[3]),
+    N(bfd_mach_m68020,  "m68k:68020", false, &arch_info_struct[4]),
+    N(bfd_mach_m68030,  "m68k:68030", false, &arch_info_struct[5]),
+    N(bfd_mach_m68040,  "m68k:68040", false, &arch_info_struct[6]),
+    N(bfd_mach_m68060,  "m68k:68060", false, &arch_info_struct[7]),
+    N(bfd_mach_cpu32,   "m68k:cpu32", false, &arch_info_struct[8]),
+    N(bfd_mach_fido,    "m68k:fido",  false, &arch_info_struct[9]),
 
     /* Various combinations of CF architecture features */
     N(bfd_mach_mcf_isa_a_nodiv, "m68k:isa-a:nodiv",
-      FALSE, &arch_info_struct[10]),
+      false, &arch_info_struct[10]),
     N(bfd_mach_mcf_isa_a, "m68k:isa-a",
-      FALSE, &arch_info_struct[11]),
+      false, &arch_info_struct[11]),
     N(bfd_mach_mcf_isa_a_mac, "m68k:isa-a:mac",
-      FALSE, &arch_info_struct[12]),
+      false, &arch_info_struct[12]),
     N(bfd_mach_mcf_isa_a_emac, "m68k:isa-a:emac",
-      FALSE, &arch_info_struct[13]),
+      false, &arch_info_struct[13]),
     N(bfd_mach_mcf_isa_aplus, "m68k:isa-aplus",
-      FALSE, &arch_info_struct[14]),
+      false, &arch_info_struct[14]),
     N(bfd_mach_mcf_isa_aplus_mac, "m68k:isa-aplus:mac",
-      FALSE, &arch_info_struct[15]),
+      false, &arch_info_struct[15]),
     N(bfd_mach_mcf_isa_aplus_emac, "m68k:isa-aplus:emac",
-      FALSE, &arch_info_struct[16]),
+      false, &arch_info_struct[16]),
     N(bfd_mach_mcf_isa_b_nousp, "m68k:isa-b:nousp",
-      FALSE, &arch_info_struct[17]),
+      false, &arch_info_struct[17]),
     N(bfd_mach_mcf_isa_b_nousp_mac, "m68k:isa-b:nousp:mac",
-      FALSE, &arch_info_struct[18]),
+      false, &arch_info_struct[18]),
     N(bfd_mach_mcf_isa_b_nousp_emac, "m68k:isa-b:nousp:emac",
-      FALSE, &arch_info_struct[19]),
+      false, &arch_info_struct[19]),
     N(bfd_mach_mcf_isa_b, "m68k:isa-b",
-      FALSE, &arch_info_struct[20]),
+      false, &arch_info_struct[20]),
     N(bfd_mach_mcf_isa_b_mac, "m68k:isa-b:mac",
-      FALSE, &arch_info_struct[21]),
+      false, &arch_info_struct[21]),
     N(bfd_mach_mcf_isa_b_emac, "m68k:isa-b:emac",
-      FALSE, &arch_info_struct[22]),
+      false, &arch_info_struct[22]),
     N(bfd_mach_mcf_isa_b_float, "m68k:isa-b:float",
-      FALSE, &arch_info_struct[23]),
+      false, &arch_info_struct[23]),
     N(bfd_mach_mcf_isa_b_float_mac, "m68k:isa-b:float:mac",
-      FALSE, &arch_info_struct[24]),
+      false, &arch_info_struct[24]),
     N(bfd_mach_mcf_isa_b_float_emac, "m68k:isa-b:float:emac",
-      FALSE, &arch_info_struct[25]),
+      false, &arch_info_struct[25]),
     N(bfd_mach_mcf_isa_c, "m68k:isa-c",
-      FALSE, &arch_info_struct[26]),
+      false, &arch_info_struct[26]),
     N(bfd_mach_mcf_isa_c_mac, "m68k:isa-c:mac",
-      FALSE, &arch_info_struct[27]),
+      false, &arch_info_struct[27]),
     N(bfd_mach_mcf_isa_c_emac, "m68k:isa-c:emac",
-      FALSE, &arch_info_struct[28]),
+      false, &arch_info_struct[28]),
     N(bfd_mach_mcf_isa_c_nodiv, "m68k:isa-c:nodiv",
-      FALSE, &arch_info_struct[29]),
+      false, &arch_info_struct[29]),
     N(bfd_mach_mcf_isa_c_nodiv_mac, "m68k:isa-c:nodiv:mac",
-      FALSE, &arch_info_struct[30]),
+      false, &arch_info_struct[30]),
     N(bfd_mach_mcf_isa_c_nodiv_emac, "m68k:isa-c:nodiv:emac",
-      FALSE, &arch_info_struct[31]),
+      false, &arch_info_struct[31]),
 
     /* Legacy names for CF architectures */
-    N(bfd_mach_mcf_isa_a_nodiv, "m68k:5200", FALSE, &arch_info_struct[32]),
-    N(bfd_mach_mcf_isa_a_mac,"m68k:5206e", FALSE, &arch_info_struct[33]),
-    N(bfd_mach_mcf_isa_a_mac, "m68k:5307", FALSE, &arch_info_struct[34]),
-    N(bfd_mach_mcf_isa_b_nousp_mac, "m68k:5407", FALSE, &arch_info_struct[35]),
-    N(bfd_mach_mcf_isa_aplus_emac, "m68k:528x", FALSE, &arch_info_struct[36]),
-    N(bfd_mach_mcf_isa_aplus_emac, "m68k:521x", FALSE, &arch_info_struct[37]),
-    N(bfd_mach_mcf_isa_a_emac, "m68k:5249", FALSE, &arch_info_struct[38]),
+    N(bfd_mach_mcf_isa_a_nodiv, "m68k:5200", false, &arch_info_struct[32]),
+    N(bfd_mach_mcf_isa_a_mac,"m68k:5206e", false, &arch_info_struct[33]),
+    N(bfd_mach_mcf_isa_a_mac, "m68k:5307", false, &arch_info_struct[34]),
+    N(bfd_mach_mcf_isa_b_nousp_mac, "m68k:5407", false, &arch_info_struct[35]),
+    N(bfd_mach_mcf_isa_aplus_emac, "m68k:528x", false, &arch_info_struct[36]),
+    N(bfd_mach_mcf_isa_aplus_emac, "m68k:521x", false, &arch_info_struct[37]),
+    N(bfd_mach_mcf_isa_a_emac, "m68k:5249", false, &arch_info_struct[38]),
     N(bfd_mach_mcf_isa_b_float_emac, "m68k:547x",
-      FALSE, &arch_info_struct[39]),
+      false, &arch_info_struct[39]),
     N(bfd_mach_mcf_isa_b_float_emac, "m68k:548x",
-      FALSE, &arch_info_struct[40]),
-    N(bfd_mach_mcf_isa_b_float_emac, "m68k:cfv4e", FALSE, 0),
+      false, &arch_info_struct[40]),
+    N(bfd_mach_mcf_isa_b_float_emac, "m68k:cfv4e", false, 0),
   };
 
 const bfd_arch_info_type bfd_m68k_arch =
-  N(0, "m68k", TRUE, &arch_info_struct[0]);
+  N(0, "m68k", true, &arch_info_struct[0]);
 
 /* Table indexed by bfd_mach_arch number indicating which
    architectural features are supported.  */
